
# q-expansion of newform 1881.2.a.d, downloaded from the LMFDB on 30 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1881
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 1], [0, 0], [1, 0], [-2, 1], [1, 0], [-2, -3], [-2, 1], [-1, 0], [3, 0], [2, -3], [-5, 1], [3, -5], [-4, -4], [6, 4], [-6, 2], [-4, -6], [3, 1], [-4, 5], [-9, 1], [11, -1], [4, 6], [-16, -1], [-2, 1], [5, 7], [1, -1], [-2, 3], [8, -4], [-10, 4], [-6, -4], [5, 1], [-18, 1], [-6, 6], [-3, -4], [2, -13], [-10, 8], [-2, 6], [5, 0], [-6, 10], [6, -9], [8, 7], [-9, 5], [-1, 13], [3, 8], [4, 2], [-6, 7], [-2, -10], [-14, -1], [-15, 1], [-12, -5], [-3, 8], [-4, -3], [-6, 0], [0, 12], [17, 4], [22, 2], [18, 6], [2, 0], [-8, -10], [6, 6], [-18, 4], [-4, 9], [2, -21], [6, -1], [6, -4], [3, 12], [27, -1], [13, -15], [4, -9], [-4, 21], [2, -15], [13, -8], [-8, 6], [-7, 2], [-14, -8], [-1, 21], [-1, 23], [17, -8], [26, 0], [18, -12], [-12, -9], [2, -18], [-14, 10], [-22, 6], [-7, -13], [-10, 3], [-5, -12], [-5, -13], [24, 5], [4, 12], [-13, -18], [-21, -8], [8, -23], [-3, -11], [-32, -3], [-10, 16], [14, -10], [-13, -15], [15, 11], [8, 4], [-12, -15], [-2, 9], [-6, -24], [2, -15], [22, -3], [4, -21], [13, -12], [-2, 4], [-20, 12], [-2, 8], [-4, -11], [-12, -7], [-12, 1], [-20, 2], [15, -6], [-25, -12], [9, 7], [19, -4], [-3, -24], [17, 6], [-2, 14], [-23, 19], [-18, -10], [-10, 8], [8, 22], [9, -12], [-10, -8], [3, 20], [-27, -12], [13, 22], [12, -1], [-26, 1], [14, 5], [19, 11], [8, -24], [-20, -10], [16, 10], [-14, 4], [36, 0], [9, -3], [-44, -6], [48, 3], [6, -23], [-23, 0], [-14, 7], [-13, 17], [-25, -11], [-14, 0], [-18, -21], [-1, -20], [-8, -28], [-24, 10], [15, -26], [42, -6], [6, 32], [-36, -10], [4, 30], [-38, 13], [28, 20], [8, 11], [-24, 3], [11, -12], [-16, -7], [-12, -11], [-13, -9], [1, 31], [-23, 7], [-20, 8], [-22, -11], [-34, -14], [-1, 6], [-2, 26], [-6, 20], [16, 16], [12, 6], [-11, 19], [-21, 4], [-26, 0], [5, -28], [18, -7], [0, 43], [-16, 4], [-26, -6], [-1, -15], [-14, -18], [9, 26], [20, -2], [2, -5], [8, 6], [48, -1], [26, 4], [19, -21], [10, 21], [-41, 3], [-14, 38], [-12, 10], [47, 5], [22, -22], [-25, 4], [-2, 3], [-6, -14], [8, -12], [54, 0], [12, -18], [12, -22], [-33, 0], [49, 4], [-9, -28], [36, -22], [16, 19], [6, -20], [12, -23], [-61, 6], [-17, -12], [28, -20], [28, -9], [19, -19], [-24, 15], [-14, -15], [-20, -6], [-47, -8], [16, 14], [-60, 2], [-21, -19], [37, -20], [-8, -4], [6, 16], [62, 4], [8, -8], [-32, 14], [-32, -7], [-21, 10], [-20, -34], [-18, 22], [-2, -21], [9, 44], [30, 26], [27, 32], [-40, -16], [57, -6], [7, 43], [51, -7], [-16, -6], [-6, 8], [-13, -13], [0, -26], [-16, -34], [68, -4], [52, 10], [-20, -1], [62, 7], [48, 4], [14, 2], [17, -30], [-26, 26], [-6, 14], [-12, -5], [12, 6], [-4, 15], [-7, -1], [38, 15], [-4, -6], [18, -21], [46, -3], [22, 0], [4, 2], [0, -22], [33, -1], [11, 46], [8, 39], [-26, -30], [15, 39], [-29, 27], [8, -14], [-32, -14], [34, -17], [-53, -6], [-7, -20], [-10, 10], [-8, 12], [16, 15], [-36, -6], [-13, -6], [24, -16], [18, -20], [-8, 39], [-27, -18], [2, -16], [22, 18], [13, -8], [8, 13], [25, 28], [66, 10], [-44, 16], [18, -20], [-18, -14], [-20, 8], [21, 19], [11, 10], [56, 9], [-46, -16], [6, 34], [34, 3], [16, 39], [-28, 36], [-23, -34], [13, -7], [27, 21], [-25, 9], [2, -20], [28, 23], [-44, 19], [-32, 38], [50, -4], [3, 41], [8, 14], [-2, -4], [-59, 6], [40, -13], [46, 0], [12, -34], [-42, -27], [-41, 8], [44, -1], [35, 7], [30, 27], [23, 19], [-27, 3], [31, -23], [14, 3], [60, -6], [-18, -27], [-36, 22], [-66, 16], [32, 13], [-24, 23], [39, 22], [-25, -17], [-4, -18], [-9, -16], [-6, -36], [-39, -3], [-57, 12], [-36, -28], [-35, 37], [47, 33], [64, 3], [67, -2], [-7, -9], [-12, -6], [11, -26], [-14, -39], [35, 21], [12, 52], [56, -8], [0, 21], [44, 31], [-41, -2], [22, 20], [-42, 16], [-11, -29], [66, 20], [2, 3], [-11, -29], [34, 4], [-49, 0], [37, 31], [-4, 15], [-40, -25], [12, -20], [29, 2], [-56, -14], [-30, -25], [-4, -26], [-18, 43], [11, 18], [-88, 0], [-6, -4], [-54, -29], [-2, 23], [10, -40], [68, 18], [39, 12], [54, 13], [-18, 9], [16, -38], [-11, 7], [-6, -46], [-9, -14], [16, -45], [-35, -7], [-41, -37], [-31, -3], [77, -6], [-3, -6], [-23, -21], [-32, 4], [-14, 0], [-2, -18], [-32, 12], [-42, -3], [38, -26], [19, -2], [0, -54], [8, -18], [22, 19], [10, 22], [4, -14], [-14, 1], [-26, 40], [-11, -26], [-78, -16], [-34, -13], [-44, 0], [18, -44], [10, -25], [19, 42], [10, -59], [-63, 7], [48, 14], [-40, -17], [5, -33], [-8, 28], [-78, -12], [34, -20], [-20, -26], [61, 15], [-77, 9], [-50, -10], [-19, -40], [-80, 14]]
